##########################################################################################################
#  Supplementary Materials: An algorithm for genetic analysis of full-sib datasets with mixed model      #
#  software lacking a pedigree function to equate male and female random effects.                        #
#  Hodge, GR & Acosta, JJ (2020)                                                                         #
#                                                                                                        #
#  R_parDouble.R: R code for parent model analysis with doubled dataset. Code completes REML estimation  #
#  of variance components, calculates genetic parameters, and predicts parental GCA (BV), progeny BV,    #
#  and full-sib family SCA effects.                                                                      #
##########################################################################################################

rm(list=ls())
setwd("/Path/to/Working/Directory/")                                                      

# Read and double the dataset
all1 <- read.csv("FullSibBLUP_Example.csv", header=T, sep=",", stringsAsFactors=T, na.strings=c("."))        
all2 <- subset(all1, select = c("treeid","male","female","group","test","rep","stvol"))
all1$class = "FM"
all2$class = "MF"
colnames(all2) = colnames(all1)

all <- rbind(all1,all2)

############################ Mixed Models for Estimation of Genetic Parameters ###########################

library(lme4)
library(arm)

out <- "Model_Solutions"
dir.create(out)

# Model Settings 
Trait = "stVOL"
C_equation <- as.formula(stvol ~ test + rep:test + (1|female) + (1|male) + (1|female:male) + 
                                 (1|female:test) + (1|male:test) + (1|female:male:test)) 

mod<-lmer(C_equation, data = all)

################################ Variance Components and Genetic Parameters ##############################

vc <- as.data.frame(VarCorr(mod), comp="Variance")[ ,-c(2:3)]
vc <- data.frame(vc[,-1],row.names = vc[,1])

vf  = vc["female",1]
vfe  = vc["female:test",1]
vfm   = vc["female:male",1]
vfme  = vc["female:male:test",1]
vresid  = vc["Residual",1]

vphen = 2*vf + vfm + 2*vfe + vfme + vresid
vadd  = 4*vf
h2    = vadd/vphen
h2w   = 2*vf/vresid
vdom  = 4*vfm 
d2    = vdom/vphen
rBg   = vf/(vf + vfe)
rBd   = vfm/(vfm + vfme)

All_cleanParms <- data.frame(Trait,vf,vfe,vfm,vfme,vresid,vphen,vadd,h2,h2w,d2,rBg,rBd)

write.table(t(All_cleanParms), paste0(out,"/R_parms.csv"),sep = ",", col.names = F)

################################################ BLUPs ###################################################

BLUP = ranef(mod) # extracting random effects solutions

############################################## Female GCAs ###############################################

femBLUPs <- BLUP$female
db.names <- data.frame(do.call(cbind, list(rownames(femBLUPs))))
names(db.names) = c("female")
femBLUPs <- data.frame(db.names,GCA = femBLUPs[ ,1], BV = 2*femBLUPs[ ,1])   

# Extracting the se(g-ghat) for the BLUP solutions from the lmer output.
seGCAhat <- se.ranef(mod)$female
db.names <- data.frame(do.call(cbind, list(rownames(seGCAhat))))
names(db.names) = c("female")
seGCAhat <- data.frame(db.names,sepBV = 2*seGCAhat[ ,1])   

GCA_BLUPs <- merge(femBLUPs,seGCAhat,by="female")
GCA_BLUPs <- GCA_BLUPs[order(GCA_BLUPs$female), ]

write.csv(GCA_BLUPs, paste0(out,"/R_GCA.csv"), row.names = F) 

############################################## SCA BLUPs ################################################# 

SCA_BLUPs <- BLUP$`female:male`
db.names <- data.frame(do.call(rbind, strsplit(rownames(SCA_BLUPs), ":", fixed=TRUE)))
names(db.names) = c("female","male")
SCA_BLUPs <- data.frame(db.names,SCA = SCA_BLUPs[,1])                                              

# Extracting the se(g-ghat) for the BLUP solutions from the lmer output.
seSCAhat <- se.ranef(mod)$`female:male`
db.names <- data.frame(do.call(rbind, strsplit(rownames(seSCAhat), ":", fixed=TRUE)))
names(db.names) = c("female","male")
seSCAhat <- data.frame(db.names,sepSCA = seSCAhat[,1])                                              

SCA_BLUPs <- merge(SCA_BLUPs,seSCAhat,by=c("female","male"))
SCA_BLUPs$female_male <- paste0(SCA_BLUPs$female,"_",SCA_BLUPs$male)
                                             
SCA_BLUPs <- subset(SCA_BLUPs, select = c("female","male","female_male","SCA","sepSCA"))
SCA_BLUPs <- SCA_BLUPs[order(SCA_BLUPs$female,SCA_BLUPs$male), ]

write.csv(SCA_BLUPs, paste0(out,"/R_SCA.csv"), row.names = F)

################################# Calcualtion of Family Genetic Value ####################################

mGCA_BLUPs <- GCA_BLUPs
names(mGCA_BLUPs)[1] = "male"

famGV <- merge(SCA_BLUPs, GCA_BLUPs, by="female")
famGV <- merge(famGV, mGCA_BLUPs, by="male")
famGV <- subset(famGV, select = c("female","male","female_male","GCA.x","GCA.y","SCA"))
names(famGV) = c("female","male","female_male","GCAfem","GCAmale","SCAfm")

famGV$FSfmGain = famGV$GCAfem + famGV$GCAmale + famGV$SCAfm

famGV <- famGV[order(famGV$female,famGV$male), ]

write.csv(famGV, paste0(out,"/R_FSfmGain.csv"), row.names = F) 

######################## Within Family selection, calculation of Individual BV ###########################

# Extracting Residuals from the lmer output and calculating within family deviation.
all$ResidR <- residuals(mod)
all$wFamDev <- all$ResidR*h2w

# Merge the female GCA, male GCA, and residual and calculate IndTreeBV.  
all <- Reduce(merge, list(all, GCA_BLUPs))

all <- merge(all, mGCA_BLUPs, by = "male")
names(all) <- c("male","female","treeid","group","test","rep","stvol","class","ResidR",   
                "GWtree","GCAfem","BVfem","sepBVfem","GCAmale","BVmale","sepBVmale")

iBV <- subset(all, select = c("treeid","female","male","group","test","rep","stvol","class","BVfem","BVmale","GWtree"))
iBV <- droplevels(iBV[-which(iBV$class == "MF"), ])
iBV$BVmid <- 0.5*(iBV$BVfem + iBV$BVmale)
iBV$BVtree <- iBV$BVmid + iBV$GWtree 
iBV <- subset(iBV, select = c("treeid","female","male","group","test","rep","stvol","BVfem","BVmale","BVmid","GWtree","BVtree"))
iBV <- iBV[order(iBV$treeid), ]

write.csv(iBV, paste0(out,"/R_BVtree.csv"), row.names = F)

######################################### Conf.Interval of VCs ###########################################

# The code below calculates CI of VarComps form the model, not a fast process, it may take several minutes to run
varvc <- confint(mod, level = 0.6827, maxpts=8)
varvc