/* ************************************************************************************************** */
/*  Supplementary Materials: An algorithm for genetic analysis of full-sib datasets with mixed model  */
/*  software lacking a pedigree function to equate male and female random effects.                    */
/*  Hodge, GR & Acosta, JJ (2020)                                                                     */
/*                                                                                                    */
/*  SAS_parDouble.sas: SAS code for parent model analysis with doubled dataset.                       */
/*  Code completes REML estimation of variance components, calculates genetic parameters, and         */
/*  predicts parental GCA (BV), progeny BV, and full-sib family SCA effects.                          */
/* ************************************************************************************************** */

libname perm 'C:\MyFiles\' extendobscounter=NO;

/*********************** SAS BLUP SAS_parDouble ***********************/

/* Import the Data */
proc import out= FullSibBLUP
  datafile= "C:\MyFiles\FullSibBLUP_Example.csv"
  dbms=CSV replace; 
  getnames=YES;
run;

/* Double the Dataset */
data FullSibBLUP; set FullSibBLUP;
class='FM'; output;
class='MF'; mom=male; dad=female; female=mom; male=dad; output;
drop mom dad;


/* PROC MIXED - REML, BLUPs */
proc mixed data=FullSibBLUP asycov;
class test rep female male;
model stvol = test rep(test) / residual outp=residuals;
random female male female*male female*test male*test female*male*test / solution;
ods output covparms=varcomps;
ods output solutionr=blups;
ods output asycov=asycov;
run;
proc print data=varcomps;
proc print data=blups;  
/*proc print data=residuals;  where treeid le 50;*/
proc print data=asycov;
run;


/* Clean Variance Components */
data f fe fm fme err; set varcomps;
if covparm='female' then do; vf=estimate; output f; end;
if covparm='female*male' then do; vfm=estimate; output fm; end;
if covparm='test*female' then do; vfe=estimate; output fe; end;
if covparm='test*female*male' then do; vfme=estimate; output fme; end;
if covparm='Residual' then do; vresid=estimate; output err; end;
data f; set f; keep vf;
data fe; set fe; keep vfe;
data fm;  set fm;  keep vfm;
data fme; set fme; keep vfme;
data err; set err; keep vresid;
data SASparms; merge f fe fm fme err;
vphen=2*vf + 2*vfe + vfm + vfme + vresid;
vadd=4*vf;
vdom=4*vfm;
h2 = vadd/vphen;
d2 = vdom/vphen;
rbg = vf / (vf+vfe);
rbd = vfm /  (vfm+vfme);
h2w = 2*vf / vresid;
proc print;
run;

/* Asymptotic Variance-Covariance Matrix for the GenParms */
data perm.SAS_asycov; set asycov;
/* Extract stderrs for varcomp estimates */
/* Remember that var and se values for vfm, vfme, and vresid are low, */
/* due to the doubling of "apparent" degrees for freedom for these terms. */
/* A very good approximate adjustment is to multiply the se(vc) by N = sqrt(2). */ 
/* This should be applied to sevfm, sevfme, sevresid. */
data sevf; set asycov;
if CovParm='female'; Varvf=CovP1; sevf=sqrt(Varvf);
data sevfm; set asycov;
if CovParm='female*male'; Varvfm=CovP3; sevfm=sqrt(Varvfm);
data sevfe; set asycov;
if CovParm='test*female'; Varvfe=CovP4; sevfe=sqrt(Varvfe);
data sevfme; set asycov;
if CovParm='test*female*male'; Varvfme=CovP6; sevfme=sqrt(Varvfme);
data sevresid; set asycov;
if CovParm='Residual'; Varvresid=CovP7; sevresid=sqrt(Varvresid);
data sevc; merge sevf sevfm sevfe sevfme sevresid;
varvadd = 16*varvf;
sevadd = 4*sevf;
keep sevadd sevf sevfm sevfe sevfme sevresid varvadd varvf varvfm varvfe varvfme varvresid;
proc print; run;

/* calculate sevphen */
data sevphen; set asycov;
sumcov=CovP1 + CovP2 + CovP3 + CovP4 + CovP5 + CovP6 + CovP7;
/* also extract Cov(vf,vfe) and Cov(vfm,vfme) to use for calc. serbg and serbd */
if CovParm='female' then covvfvfe=CovP4;
if CovParm='female*male' then covvfmvfme=CovP6;
proc means noprint data=sevphen;
var sumcov covvfvfe covvfmvfme;
output out=sevphen sum=varvphen covvfvfe covvfmvfme;
data sevphen; set sevphen;
 sevphen=sqrt(varvphen);
 drop _type_ _freq_;
proc print; run;
data sevc; merge sevc sevphen;
proc print; run;


/* Merge Varvc and sevc onto parms dataset */
data SASparms; merge SASparms sevc;
seh2 = 4*sevf/vphen;
sed2 = 4*sevfm/vphen;
seh2w = 2*sevf/vresid;
serbg = sqrt(rbg**2 * (varvf/vf**2 + (varvf+varvfe+2*Covvfvfe)/(vf+vfe)**2 - 2*(varvf+covvfvfe)/(vf * (vf+vfe)))); 
serbd = sqrt(rbd**2 * (varvfm/vfm**2 + (varvfm+varvfme+2*Covvfmvfme)/(vfm+vfme)**2 - 2*(varvfm+covvfmvfme)/(vfm * (vfm+vfme)))); 
proc print; 
var h2 seh2 d2 sed2 h2w seh2w rbg serbg rbd serbd; run;


/* Clean Parms for easy reading */
/* create data set with Parameter, estimate, stderr */
/* stderradj is calculated for parms where the doubled-dataset produces artificially low standard errors */
/* stderradj = stderr*sqrt(2), which is satisfactory for any dataset of reasonable size */
data perm.SAS_parms; set SASparms;
Parameter='vf    '; estimate=vf;     stderr=sevf;     stderradj=.;              output;
Parameter='vfe   '; estimate=vfe;    stderr=sevfe;    stderradj=.;              output;
Parameter='vfm   '; estimate=vfm;    stderr=sevfm;    stderradj=stderr*sqrt(2); output;
Parameter='vfme  '; estimate=vfme;   stderr=sevfme;   stderradj=stderr*sqrt(2); output;
Parameter='vresid'; estimate=vresid; stderr=sevresid; stderradj=stderr*sqrt(2); output;
Parameter='vphen '; estimate=vphen;  stderr=sevphen;  stderradj=stderr*sqrt(2); output;
Parameter='vadd  '; estimate=vadd;   stderr=sevadd;   stderradj=.;              output;
Parameter='h2    '; estimate=h2;     stderr=seh2;     stderradj=.;              output;
Parameter='h2w   '; estimate=h2w;    stderr=seh2w;    stderradj=.;              output;
Parameter='d2    '; estimate=d2;     stderr=sed2;     stderradj=stderr*sqrt(2); output;
Parameter='rBg   '; estimate=rBg;    stderr=serBg;    stderradj=.;              output;
Parameter='rBd   '; estimate=rBd;    stderr=serBd;    stderradj=stderr*sqrt(2); output;
keep parameter estimate stderr stderradj;
proc print; run;


/* Residual dataset for calculation of within-family genetic values */ 
data GW; set residuals;
keep treeid class female male group test rep stvol resid;

/* Parent GCA and BV */
data GCA; set blups;
if effect='female';
GCA=estimate;
BV=2*GCA;
sepBV=2*StdErrPred;
keep female GCA BV sepBV;
proc print; run;
proc means; run;

/* Family SCA */
data SCA; set blups;
if effect='female*male';
female_male=compress(female||"_"||male);
SCA = estimate;
sepSCA = StdErrPred;
keep female male female_male SCA sepSCA;
proc print;
proc means; run;

/* Full-Sib Family Genetic Value = (GCAmom+GCAdad)/2 + SCA */
proc sort data=SCA; by female;
proc sort data=GCA; by female;
data FSfmGain; merge SCA GCA; by female;
GCAfem=GCA;
keep female male female_male SCA GCAfem;

proc sort data=FSfmGain; by male;
data GCAmale; set GCA;
male=female; drop female;
proc sort data=GCAmale; by male;
data FSfmGain; merge FSfmGain GCAmale; by male;
GCAmale=GCA;
SCAfm=SCA;
FSfmGain = GCAfem + GCAmale + SCAfm;
keep female male female_male SCAfm GCAfem GCAmale FSfmGain;
proc print;
run;


/* Create TreeBV for SAS_parDouble */
data female; set GCA;
BVfem = BV;
keep female BVfem;

data male; set GCA;
BVmale = BV;
male = female;
keep male BVmale;

proc sort data=GW; by female;
proc sort data=female; by female;
data GW; merge GW female; by female;

proc sort data=GW; by male;
proc sort data=male; by male;
data SAS_BVtree; merge GW male; by male;
dummy=999;
data h2w; set SASparms;
dummy=999; keep dummy h2w;

data SAS_BVtree; merge SAS_BVtree h2w; by dummy;
BVmid=(BVfem + BVmale)/2;
GWtree = h2w*resid;
BVtree = BVmid + GWtree;
proc print; where treeid le 50;
var treeid female male test rep BVfem BVmale BVmid BVtree GWtree h2w;
run;

/* Delete the doubled observations */
data perm.SAS_BVtree; set SAS_BVtree;
if class='MF' then delete;
drop dummy class resid h2w;
data perm.SAS_GCA; set GCA;
data perm.SAS_SCA; set SCA;
data perm.SAS_FSfmGain; set FSfmGain;
run;

/* Export to CSV files */
proc export data=perm.SAS_BVtree 
 outfile="C:\MyFiles\SAS_BVtree.csv" 
 dbms=csv replace; 
proc export data=perm.SAS_GCA 
 outfile="C:\MyFiles\SAS_GCA.csv" 
 dbms=csv replace; 
proc export data=perm.SAS_SCA 
 outfile="C:\MyFiles\SAS_SCA.csv" 
 dbms=csv replace; 
proc export data=perm.SAS_FSfmGain
 outfile="C:\MyFiles\SAS_FSfmGain.csv" 
 dbms=csv replace; 
proc export data=perm.SAS_parms 
 outfile="C:\MyFiles\SAS_parms.csv" 
 dbms=csv replace; 
run;
